@@ -121,8 +121,8 @@ def get_value_from_table(csv_file, col_referer=0, col_return=1,
 
     return _get_value_from_table
 
-def get_ids(args, model, fields, op='&'):
-    """Return ``model`` which ``fields``'s value matched ``args[3]`` value
+def get_ids(model, fields, op='&'):
+    """Return ``model`` which one or more ``fields`` matched column ``value``
    
     >>> 'partner_ids': Column(56, get_ids('res.partner.address', ['street', 'street2'], op='|'))
     [100, 102]
@@ -139,7 +139,6 @@ def get_ids(args, model, fields, op='&'):
 
     """
     def _get_ids(self, b_model, b_field, b_value, b_line):
-        print self, b_model, b_field, b_value, b_line
         search = []
         try:
             b_value = str(b_value).strip()
@@ -151,19 +150,21 @@ def get_ids(args, model, fields, op='&'):
                 search.append(op)
                 i += 1
             search.append((field, '=', b_value))
-        print search
 
-        res = self.server.execute(self.db, self.uid, self.pwd,
-               model, 'search', search) 
+        try:
+            res = self.server.execute(self.db, self.uid, self.pwd,
+               model, 'search', search)
+        except RuntimeError:
+            return self._get_ids(b_model, b_field, b_value, b_line)
         if not res:
-            self._logger.warning(u'get_ids(%s, %s): no object found for value `%s` on model' %
-                    (model, fields, b_value), extra=STATS)
+            self._logger.error(u"'%s' not found in fields %s from %s" %
+                    (b_value, fields, model), extra=STATS[self._id])
         return res
     
     return _get_ids
 
 def get_id(model, fields, op='&'):
-    """Return ``model`` which ``fields``'s value matched ``args[3]`` value
+    """Return ``model`` which one or more ``fields`` matched column ``value``
    
     >>> 'address_ids': (21, get_ids('res.partner', ['name']))
     [99]
@@ -185,16 +186,85 @@ def get_id(model, fields, op='&'):
     
     return _get_id
 
+def get_objects(model, fields, op='&'):
+    """Return ``model`` instance which ``fields``'s value matched ``value``
+    
+    >>> 'parent_name': Column(2, get_objects('res.partner.address', ['ref'])[0]['name'])
+    ['Toto']
+    
+    :param model: The name of the model to search into
+    :type model: str
+    :param fields: The name of the model's field
+    :type fields: str
+    :param value: The value to compare to
+    :type value: str
+    :returns: list
 
-def to_boolean(mode='on',
+    """
+
+    def _get_objects(self, b_model, b_field, b_value, b_line):
+        return self.server.execute(self.db, self.uid, self.pwd,
+               model, 'read', get_id(model, fields, op))
+
+    return _get_objects
+
+def get_object(model, fields, field, op='&'):
+    """Return ``model`` instance which ``fields``'s value matched ``value``
+    
+    >>> 'parent_name': Column(2, get_object('res.partner.address', ['ref'])['name'])
+    ['Toto']
+    
+    :param model: The name of the model to search into
+    :type model: str
+    :param fields: The name of the model's field
+    :type fields: str
+    :param field: The desire field's value to returned
+    :type field: str
+    :param value: The value to compare to
+    :type value: str
+    :returns: dict
+
+    """
+    
+    def _get_object(*args):
+        res = get_objects(model, fields, op=op)(*args)
+        res = res and res[0]
+        return res and field in res and res[field]
+
+    return _get_object
+
+def get_fields(model, fields, op='&'):
+    """Return ``model`` instance which one or more ``fields`` matched column ``value``
+    
+    >>> 'parent_name': Column(2, get_fields('res.partner.address', ['ref'])[0]['name'])
+    'Toto'
+    
+    :param model: The name of the model to search into
+    :type model: str
+    :param fields: The name of the model's field
+    :type fields: str
+    :param value: The value to compare to
+    :type value: str
+    :returns: list
+
+    """
+
+    def _get_fields(self, b_model, b_field, b_value, b_line):
+        ids = get_ids(model, fields, op)(self, b_model, b_field, b_value, b_line)
+        res = self.server.execute(self.db, self.uid, self.pwd, model, 'read', ids)
+        return res
+
+    return _get_fields
+
+def to_boolean(mode=None,
         true_values=('O', 'Y', '1', 'o', 'y', 'true', 'True'),
         false_values=('0', 'N', '2', 'n', 'false', 'False')):
     """Return a boolean depending from the input value.
     
-    >>> 'field': Column(2, to_boolean()
+    >>> 'field': Column(2, to_boolean())
     
-    :param mode: lambda args (ie: [self, model ,field, content, line])
-    :type mode: list
+    :param mode: First char will be used if value is True and second if False
+    :type mode: str
     :param true_values: The name of the model to search into
     :type true_values: str
     :param false_values: The name of the model's field
@@ -211,9 +281,15 @@ def to_boolean(mode='on',
 
     """
     def _to_boolean(self, model, field, value, line):
-        return (value in true_values and True)\
-                or (value in false_values and False)\
-                or (value and True or False)
+        if mode:
+            true = mode[0]
+            false = mode[1]
+        else:
+            true = True
+            false = False
+        return (value in true_values and true)\
+                or (value in false_values and false)\
+                or (value and true or false)
 
     return _to_boolean
 
@@ -238,3 +314,40 @@ def to_date():
 
     return _to_date
 
+def city_cedex(res=None, cedex_code=False):
+    """Check if a cedex code is in the town name.
+    Return the relative string to ``res`` argument
+
+    >>> 'city':  Column(6, city_cedex('city'))
+    >>> 'cedex': Column(6, city_cedex('cedex'))
+    
+    :param res: Relative string returned
+    :type res: str
+    :param cedex_code: Returns only code or full code ('6', 'CEDEX 6')
+    :type cedex_code: str
+    :returns: str or tuple
+
+    """
+    def _city_cedex(self, model, field, value, line):
+        string = value
+        string.upper()
+        cedex = ""
+
+        if string.count('CEDEX'):
+            string = string.split('CEDEX')[0].strip(' ')
+            try:
+                cedex = "CEDEX %s" % int(string.split('CEDEX')[1].strip(' '))
+            except IndexError:
+                cedex = "CEDEX"
+            except ValueError:
+                cedex = 'CEDEX'
+        if res == 'city':
+            return string
+        elif res == 'cedex':
+            return cedex
+        else:
+            return (string, cedex)
+
+    return _city_cedex
+
+
